/***************************************************************/
/*                                                             */
/*  CONFIG.H                                                   */
/*                                                             */
/*  Contains various configuration parameters for Remind.      */
/*  You may have to edit this file to tweak parameters or take */
/*  care of certain system dependencies.                       */
/*                                                             */
/*  This file is part of REMIND.                               */
/*  Copyright (C) 1991 by David F. Skoll.                      */
/*                                                             */
/***************************************************************/

/*---------------------------------------------------------------------*/
/* STRSTR:  If your system does not have the "strstr" function,        */
/* uncomment the following line.                                       */
/*---------------------------------------------------------------------*/
/* #define NO_STRSTR 1 */

/*---------------------------------------------------------------------*/
/* STDLIB:  If you don't have the <stdlib.h> header file, comment the  */
/* following line.                                                     */
/*---------------------------------------------------------------------*/
#define HAVE_STDLIB_H 1

/*---------------------------------------------------------------------*/
/* MALLOC:  If you do not have the <malloc.h> header file,             */
/* comment out the next 3 lines.                                       */
/*---------------------------------------------------------------------*/
#ifdef UNIX
#define HAVE_MALLOC_H 1
#endif

/*---------------------------------------------------------------------*/
/* BASE: The base year for date calculation.  NOTE!  January 1 of the  */
/*       base year MUST be a Monday, else Remind will not work!        */
/*---------------------------------------------------------------------*/
#define BASE 1990

/*---------------------------------------------------------------------*/
/* YR_RANGE: The range of years allowed.  Computers with 16-bit        */
/*           integers can handle about 89 years worth of reminders; if */
/*           you use 32-bit integers, you can handle over 5 867 000    */
/*           years.  Note that YR_RANGE is set to 88 rather than 89    */
/*           because we can range up to the last day of the 88th year. */
/*---------------------------------------------------------------------*/
#define YR_RANGE 88

/*---------------------------------------------------------------------*/
/* VAR_NAME_LEN: The maximum length of variable names                  */
/*---------------------------------------------------------------------*/
#define VAR_NAME_LEN 12

/*---------------------------------------------------------------------*/
/* MAX_PRT_LEN: The maximum number of characters to print when         */
/* displaying a string value for debugging purposes.                   */
/*---------------------------------------------------------------------*/
#define MAX_PRT_LEN 40

/*---------------------------------------------------------------------*/
/* LINELEN: The maximum length of an input line                        */
/*---------------------------------------------------------------------*/
#define LINELEN 512

/*---------------------------------------------------------------------*/
/* OP_STACK_SIZE: The size of the operator stack for expr. parsing     */
/*---------------------------------------------------------------------*/
#define OP_STACK_SIZE 30

/*---------------------------------------------------------------------*/
/* VAL_STACK_SIZE: The size of the operand stack for expr. parsing     */
/*---------------------------------------------------------------------*/
#define VAL_STACK_SIZE 30

/*---------------------------------------------------------------------*/
/* INCLUDE_NEST: How many nested INCLUDES do we handle?                */
/*---------------------------------------------------------------------*/
#define INCLUDE_NEST 8

/*---------------------------------------------------------------------*/
/* IF_NEST: How many nested IFs do we handle?  Maximum is the number   */
/* of bits in an int, divided by two.  Beware!                         */
/*---------------------------------------------------------------------*/
#define IF_NEST (4*sizeof(unsigned int))

/*---------------------------------------------------------------------*/
/* Do we handle queued reminders?                                      */
/*---------------------------------------------------------------------*/
#ifdef UNIX
#define HAVE_QUEUED 1
#endif

/*---------------------------------------------------------------------*/
/* Does our C compiler have prototypes?  Override this test if you     */
/* are using a non-ANSI compiler that nevertheless has prototypes.     */
/*---------------------------------------------------------------------*/
#ifdef __STDC__
#define HAVE_PROTOS 1
#endif

/*---------------------------------------------------------------------*/
/* Do we use the <stdarg.h> scheme for functions with variable number  */
/* of parameters?  If not, the <varargs.h> scheme is assumed.          */
/*---------------------------------------------------------------------*/
#ifdef __STDC__
#define HAVE_STDARG 1
#endif

/*---------------------------------------------------------------------*/
/* Do we have the <unistd.h> header?  If not, use sys/files.h          */
/*---------------------------------------------------------------------*/
#define HAVE_UNISTD 1

/*---------------------------------------------------------------------*/
/* How many attempts to resolve a weird date spec?                     */
/*---------------------------------------------------------------------*/
#define TRIG_ATTEMPTS 25

/*---------------------------------------------------------------------*/
/* How many global omits of the form YYYY MM DD do we handle?          */
/*---------------------------------------------------------------------*/
#define MAX_FULL_OMITS 75

/*---------------------------------------------------------------------*/
/* How many global omits of the form MM DD do we handle?               */
/*---------------------------------------------------------------------*/
#define MAX_PARTIAL_OMITS 75

/*---------------------------------------------------------------------*/
/* The size of statically-allocated buffers for tokens.                */
/*---------------------------------------------------------------------*/
#define TOKSIZE 128

/*---------------------------------------------------------------------*/
/* The size of the buffer for the shell() function.                    */
/*---------------------------------------------------------------------*/
#define SHELLSIZE 512

/*---------------------------------------------------------------------*/
/* A newline - some systems need "\n\r"                                */
/*---------------------------------------------------------------------*/
#define NL "\n"

/*---------------------------------------------------------------------*/
/* Minimum number of linefeeds in each calendar "box"                  */
/*---------------------------------------------------------------------*/
#define CAL_LINES 5

/*---------------------------------------------------------------------*/
/* Don't change the next definitions                                   */
/*---------------------------------------------------------------------*/
#define PUBLIC
#define PRIVATE static
